#!/bin/bash

CONTROLLER_USER="electrosense"
CONTROLLER_HOST="controller.electrosense.org"
CONTROLLER_PORT_SSH="2227"
CONTROLLER_PORT_MYSQL="3308"
INTERFACE="eth0"

# SSL
SSL_DIR="/etc/electrosense/certs"
SSL_CA="$SSL_DIR/CA-Cert.pem"
SSL_CERT="$SSL_DIR/Sensor-MySQL-Cert.pem"
SSL_KEY="$SSL_DIR/Sensor-MySQL-SK.pem"

# Check the tunnel is up each this many seconds
# This value has to be smaller than UPDATE_DB
STATUS_TIMER=10

# Update database info each 5 min
DB_TIMER=300

# COUNTER_DB initialised at max so DB update is carried on first iteration
DB_COUNTER=0


###############################################

MAC_HEX=$(ip link show $INTERFACE | awk '/ether/ {print $2}')
MAC_HEX=${MAC_HEX//:}
MAC_DEC=$((16#$MAC_HEX))

echo "Management started for serial $MAC_DEC (${MAC_HEX})"

while true
do
  TIMESTAMP=$(date)

  # Admit only one connection
  PROC=`pgrep -f '0:localhost:22' | wc -l`

  # if there is no session running or too many
  if [ "$PROC" -ne 1 ]
  then
    # Kill extra sessions
    if [ "$PROC" -gt 1 ]
    then
      echo "$TIMESTAMP  Pre-emptively killing $PROC SSH sessions."
      pgrep -f '0:localhost:22' | xargs kill
    fi

    # SSH
    PORT=`ssh -o StrictHostKeyChecking=no -o TCPKeepAlive=yes -o ServerAliveInterval=60 -o ServerAliveCountMax=3 -i /etc/electrosense/certs/id_sensor -fNT -R 0:localhost:22 -p $CONTROLLER_PORT_SSH $CONTROLLER_USER@$CONTROLLER_HOST 2>&1 | grep -m 1 -o "Allocated port [0-9]\+" | grep -o "[0-9]\+"`

    if [ "$PORT" == "" ]
    then
      printf "$TIMESTAMP  Failed to open reverse SSH tunnel_ RC $?.\n"
    else
      printf "$TIMESTAMP  Reverse SSH tunnel opened successfully on remote port $PORT.\n"
      # Force update
      PROC=1
      DB_COUNTER=$DB_TIMER
    fi
  fi

  # Only if we have a valid connection
  if [ "$PROC" -eq 1 -a  $DB_COUNTER -ge $DB_TIMER ]
  then
    DB_COUNTER=0
    #MySQL
    SQL_REQ="REPLACE INTO local_connections(MAC,Port) VALUES($MAC_DEC, $PORT)"
    SQL_SSL="--ssl-ca=$SSL_CA --ssl-cert=$SSL_CERT --ssl-key=$SSL_KEY"
    SQL_RES=`mysql --host="$CONTROLLER_HOST" --port=$CONTROLLER_PORT_MYSQL --user=sensor --password=es-sensor-pass --database=electrosense $SQL_SSL --execute="$SQL_REQ"`
    if [ -z $SQL_RES ]; then
      printf "$TIMESTAMP  MySQL update succeeded for $MAC_DEC.\n"
    else
      pkill -f '0:localhost:22'
      printf "$TIMESTAMP MySQL update failed. Reverse SSH tunnel closed.\n"
    fi
  fi

  # LOOP
  sleep $STATUS_TIMER
  DB_COUNTER=$(( DB_COUNTER + STATUS_TIMER ))
done
